package org.apidesign.apifest08.test;

import static org.apidesign.apifest08.test.Currencies.CZK;
import static org.apidesign.apifest08.test.Currencies.SKK;
import static org.apidesign.apifest08.test.Currencies.USD;

import java.math.BigDecimal;

import junit.framework.TestCase;

import org.apidesign.apifest08.currency.Amount;
import org.apidesign.apifest08.currency.ConversionException;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.UnsupportedConversionException;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor createCZKtoUSD() {
        return new Convertor(new BigDecimal(17), USD, CZK);
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor createSKKtoCZK() {
    	return new Convertor(new BigDecimal("0.8"), SKK, CZK);
    }
    
    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        Convertor c = createCZKtoUSD();
        // convert $5 to CZK using c:
        Amount result =  c.convert(new BigDecimal(5), USD, CZK);
        assertEquals("Result is 85 CZK", 85, result.getValue().intValue());

        // convert $8 to CZK
        result =  c.convert(new BigDecimal(8), USD, CZK);        
        assertEquals("Result is 136 CZK", 136, result.getValue().intValue());

        // convert 1003CZK to USD
        result =  c.convert(new BigDecimal(1003), CZK, USD);
        assertEquals("Result is 59 USD", 59, result.getValue().intValue());
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        Convertor c = createSKKtoCZK();
        // convert 16CZK using c:
        Amount result =  c.convert(new BigDecimal(16), CZK, SKK);
        assertEquals("Result is 20 SKK", 20, result.getValue().intValue());

        // convert 500SKK to CZK
        result =  c.convert(new BigDecimal(500), SKK, CZK);
        assertEquals("Result is 400 CZK", 400, result.getValue().intValue());
    }
    
    
    /**
     *  Verify that the CZK to USD convertor knows nothing about SKK.
     */
     public void testCannotConvertToSKKwithCZKUSDConvertor() throws Exception {
    	 Convertor c = createCZKtoUSD();
    	 // convert $5 to SKK, the API shall say this is not possible
    	 try {
    		 c.convert(new BigDecimal(5), USD, SKK);
    		 fail("convert $5 to SKK, the API shall say this is not possible");
    	 } catch (ConversionException e) {
    		 //expected
    	 }
    	 
    	 // convert 500 SKK to CZK, the API shall say this is not possible
    	 
    	 try {
    		 c.convert(new BigDecimal("500"), SKK, CZK);
    		 fail("convert 500 SKK to CZK, the API shall say this is not possible");
    	 } catch (ConversionException e) {
    		 //expected
    	 }
     }
    
    /** 
     * Verify that the CZK to SKK convertor knows nothing about USD.
     */
    public void testCannotConvertToSKKwithCZKSKKConvertor() throws Exception {
    	Convertor c = createSKKtoCZK();
    	// convert $5 to SKK, the API shall say this is not possible
    	try {
    		c.convert(new BigDecimal(5), USD, SKK);
    		fail("convert $5 to SKK, the API shall say this is not possible");
    	} catch(ConversionException e) {
    		//expected
    	}
    	
    	try {
    		c.convert(new BigDecimal(500), CZK, USD);	
    		fail("convert 500 CZK to USD, the API shall say this is not possible");
    	} catch(ConversionException e) {
    		//expected
    	}
    } 
}

